package de.scrum_master.app;

import static java.lang.System.currentTimeMillis;

import de.scrum_master.aspect.RunnablesCallablesAspect;
import de.scrum_master.aspect.RunnablesCallables;

public class Application {

  private static class InnerStaticRunnable implements Runnable {
    @Override
    public void run() {
//      System.out.println("Running inner static runnable");
    }
  }

  private class InnerRunnable implements Runnable {
    @Override
    public void run() {
//      System.out.println("Running inner runnable");
      new Thread(new SomeRunnable()).start();
    }
  }

  public void doSomething() {
    new Thread(new InnerRunnable()).start();
    new Thread(new SomeRunnable()).start();
  }

  private static void checkCounter(Class<?> clazz, long expectedCount) {
    long instanceCounter = useOriginalAspect ? RunnablesCallables.getCounter(clazz) : RunnablesCallablesAspect.getInstanceCounter(clazz);
    if (instanceCounter != expectedCount)
      System.err.println(
        "Unexpected counter value for class " + clazz.getName() +
        ": expected="+ expectedCount +
        ", actual=" + instanceCounter
      );
  }

  private static boolean useOriginalAspect = true;

  public static void main(String[] args) throws InterruptedException {
    final int MAX_COUNT = 1;

    if (useOriginalAspect) {
      RunnablesCallables.ACTIVE = true;
      RunnablesCallablesAspect.ACTIVE = false;
    }
    else {
      RunnablesCallables.ACTIVE = false;
      RunnablesCallablesAspect.ACTIVE = true;
    }

    long startTime = currentTimeMillis();
    for (int i = 0; i < MAX_COUNT; i++) {
      new Thread(new SomeRunnable()).start();
      new Thread(new Application.InnerStaticRunnable()).start();
      new Application().doSomething();
      new Thread(new RunnableBase()).start();
    }
    Thread.sleep(250); // Unelegant way to wait for threads to finish
    System.out.printf("Duration (minus sleeping time) = %d ms%n", currentTimeMillis() - startTime - 250);

    checkCounter(Application.InnerStaticRunnable.class, MAX_COUNT);
    checkCounter(Application.InnerRunnable.class, MAX_COUNT);
    checkCounter(SomeRunnable.class, MAX_COUNT * 3);
    checkCounter(RunnableBase.class, MAX_COUNT);
    int totalInstanceCount = useOriginalAspect ? RunnablesCallables.instanceIDs.size() : RunnablesCallablesAspect.instanceIDs.size();
    if (totalInstanceCount != MAX_COUNT * 6)
      System.err.println(
        "Unexpected number of total IDs" +
        ": expected=" + MAX_COUNT * 6 +
        ", actual=" + totalInstanceCount
      );
  }
}
