package de.scrum_master.aspect;

import static java.lang.Thread.currentThread;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentHashMap;

public privileged aspect RunnablesCallables {
  public static boolean ACTIVE = true;
  public static Map<java.lang.Class, ThreadLocal<Long>> _counters = new ConcurrentHashMap<>();
  public static final Set<String> instanceIDs = Collections.synchronizedSet(new HashSet<>());

  public static long getCounter(java.lang.Class clazz) {
    ThreadLocal<Long> local = _counters.get(clazz);
    if (local == null) { // need to initialize the ThreadLocal
      synchronized (clazz) { // synchronize on the given clazz is enough
        // Check again to avoid multi threads entered the previous if statement
        if (local == null) {
          local = new ThreadLocal<Long>() {
            @Override
            protected Long initialValue() {
              return (long) 0;
            }
          };
          _counters.put(clazz, local);
        }
      }
    }
    return (long) local.get();
  }

  private static void setCounter(java.lang.Class clazz, long counter) {
    ThreadLocal<Long> local = _counters.get(clazz);
    local.remove();
    local.set(counter);
  }

  public static long incrementCounter(java.lang.Class clazz) {
    long oldInstanceCounter = getCounter(clazz);
    setCounter(clazz, oldInstanceCounter + 1);
    return oldInstanceCounter;
  }

  after() returning(InstrumentedRunnableCallable runnableCallable) :
    if(ACTIVE) &&
    call(InstrumentedRunnableCallable+.new(..))
  {
    long oldInstanceCounter = incrementCounter(runnableCallable.getClass());
    runnableCallable.setId(oldInstanceCounter);
    runnableCallable.setCreatorTid(currentThread().getId());
    instanceIDs.add(runnableCallable.getClass().toString() + "_" + runnableCallable.getId());
    printLogMessage("[Create Runnable/Callable]", runnableCallable);
  }

  Object around(InstrumentedRunnableCallable runnableCallable) :
    if(ACTIVE) &&
    this(runnableCallable) &&
    (execution(void Runnable+.run(..)) || execution(* Callable+.call(..)))
  {
    // printLogMessage("[Before Runnable.run() / Callable.call()]",
    // runnableCallable);
    Object result = proceed(runnableCallable);
    // printLogMessage("[After Runnable.run() / Callable.call()]",
    // runnableCallable);
    return result;
  }

  private void printLogMessage(String prefix, InstrumentedRunnableCallable runnableCallable) {
    System.out.printf("%s TID=%d, %s%n", prefix, Thread.currentThread().getId(), runnableCallable.toStringIRC());
  }

}
