package de.scrum_master.aspect;

import static java.lang.Thread.currentThread;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentHashMap;

public privileged aspect RunnablesCallablesAspect {
  public static boolean ACTIVE = false;
  private static final Map<Class<?>, Long> instanceCounters = new ConcurrentHashMap<>();
  public static final Set<String> instanceIDs = Collections.synchronizedSet(new HashSet<>());

  public static synchronized long getInstanceCounter(Class<?> clazz) {
    Long counter = instanceCounters.get(clazz);
    return counter == null ? 0 : counter;
  }

  private static synchronized long incrementInstanceCounter(Class<?> clazz) {
    long oldInstanceCounter = getInstanceCounter(clazz);
    instanceCounters.put(clazz, oldInstanceCounter + 1);
    return oldInstanceCounter;
  }

  after() returning(InstrumentedRunnableCallable runnableCallable) :
    if(ACTIVE) &&
    call(InstrumentedRunnableCallable+.new(..))
  {
    long oldInstanceCounter = incrementInstanceCounter(runnableCallable.getClass());
    runnableCallable.setId(oldInstanceCounter);
    runnableCallable.setCreatorTid(currentThread().getId());
    String instanceID = runnableCallable.getClass().toString() + "_" + runnableCallable.getId();
    instanceIDs.add(instanceID);
    printLogMessage("[Create Runnable/Callable]", runnableCallable);
  }

  Object around(InstrumentedRunnableCallable runnableCallable):
    if(ACTIVE) &&
    this(runnableCallable) &&
    (execution(void Runnable+.run(..)) || execution(* Callable+.call(..)))
  {
//    printLogMessage("[Before Runnable.run() / Callable.call()]", runnableCallable);
    Object result = proceed(runnableCallable);
//    printLogMessage("[After  Runnable.run() / Callable.call()]", runnableCallable);
    return result;
  }

  private void printLogMessage(String prefix, InstrumentedRunnableCallable runnableCallable) {
    System.out.printf("%s TID=%d, %s%n", prefix, currentThread().getId(), runnableCallable.toStringIRC());
  }
}
